import { Controller, Get } from '@nestjs/common';
import { ApiTags } from '@nestjs/swagger';
import { HealthCheck, HealthCheckService, HttpHealthIndicator, TypeOrmHealthIndicator } from '@nestjs/terminus';
import { SkipThrottle } from '@nestjs/throttler';

@Controller('health')
@ApiTags('Health')
@SkipThrottle()
export class HealthController {
  constructor(
    private readonly healthCheckService: HealthCheckService,
    private readonly databaseHealthIndicator: TypeOrmHealthIndicator,
    private readonly httpHealthIndicator: HttpHealthIndicator,
  ) {}

  @Get()
  @HealthCheck()
  checkHealth() {
    return this.healthCheckService.check([]);
  }

  @Get('details')
  @HealthCheck()
  checkHealthDetails() {
    const healthIndicators = [
      () => this.databaseHealthIndicator.pingCheck('database'),
      // add your own health indicators here
    ];

    return this.healthCheckService.check(healthIndicators);
  }
}
